#include "Model.h"

#include <stdlib.h>
void BuildModel()
{

 // CClockScene compScene;
	sgGetScene()->Clear();
	HouseModel hm;
	hm.Build(20, 1000, 800);

}

#define _USE_MATH_DEFINES
#include <math.h>

SG_VECTOR xAxe = {1, 0, 0};
SG_VECTOR yAxe = {0, 1, 0};
SG_VECTOR zAxe = {0, 0, 1};
SG_POINT zeroP = {0, 0, 0};

void   HouseModel::BuildLogSection(sgFloat log_size)
{
	double   sinCalc = sin(45.0*M_PI/180.0);
	double   cosCalc = cos(45.0*M_PI/180.0);
	SG_POINT arc1BeginP = {log_size*cosCalc,  -log_size*sinCalc, 0};
	SG_POINT arc1EndP = {-log_size*cosCalc, -log_size*sinCalc, 0};
	SG_POINT arc1MidP = {0, log_size, 0};

	SG_ARC arc1;
	arc1.FromThreePoints(arc1BeginP, arc1EndP, arc1MidP, false);

	SG_ARC arc2;
	arc2.FromBeginEndNormalRadius(arc1BeginP, arc1EndP, zAxe, log_size, false);

	m_logs_shifts = -arc2.center.y;

	sgCObject*  acrsObjs[2];
	acrsObjs[0] = sgCArc::Create(arc1);
	acrsObjs[1] = sgCArc::Create(arc2);

	m_log_section = sgCContour::CreateContour(acrsObjs, 2);
}

sgC3DObject*   HouseModel::BuildLog(sgFloat logH, bool withGroove, sgFloat groove_size, sgFloat grooveShift)
{
	SG_VECTOR  extrVect = {0,0, logH};
	sgCObject*  logObj = sgKinematic::Extrude(*m_log_section, NULL, 0, extrVect, true);

	if (withGroove)
	{
		sgFloat cylH = 10*groove_size;
		sgC3DObject* cyl1 = sgCreateCylinder(groove_size, cylH, 24);
		cyl1->InitTempMatrix()->Rotate(zeroP, yAxe, 90.0*M_PI/180.0);
		SG_VECTOR transVect = {-cylH/2, -groove_size, grooveShift};
		cyl1->GetTempMatrix()->Translate(transVect);
		cyl1->ApplyTempMatrix();
		cyl1->DestroyTempMatrix();

		sgC3DObject* cyl2 = sgCreateCylinder(groove_size, cylH, 24);
		cyl2->InitTempMatrix()->Rotate(zeroP, yAxe, 90.0*M_PI/180.0);
		transVect.z  = logH-grooveShift;
		cyl2->GetTempMatrix()->Translate(transVect);
		cyl2->ApplyTempMatrix();
		cyl2->DestroyTempMatrix();
		
		sgCGroup* boolRes1 = sgBoolean::Sub((const sgC3DObject&)(*logObj), *cyl1);
		int ChCnt = boolRes1->GetChildrenList()->GetCount();

		sgCObject**  allChilds = (sgCObject**)malloc(ChCnt*sizeof(sgCObject*));
		boolRes1->BreakGroup(allChilds);
		
		sgCObject::DeleteObject(boolRes1);
		sgCObject::DeleteObject(logObj);
		logObj = allChilds[0];
		sgCObject::DeleteObject(cyl1);


		boolRes1 = sgBoolean::Sub((const sgC3DObject&)(*logObj), *cyl2);
		ChCnt = boolRes1->GetChildrenList()->GetCount();

		allChilds = (sgCObject**)malloc(ChCnt*sizeof(sgCObject*));
		boolRes1->BreakGroup(allChilds);

		sgCObject::DeleteObject(boolRes1);
		sgCObject::DeleteObject(logObj);
		logObj = allChilds[0];
		sgCObject::DeleteObject(cyl2);

		free(allChilds);
	}

	return ((sgC3DObject*)logObj);
}

const int logsCount = 20;
const int inRoofCount = 20;
const int groove_shift_count = 3;

void  HouseModel::BuildWall(int wallIndex, sgFloat log_size, sgFloat houseSizeX, sgFloat houseSizeY)
{
	
	
	switch (wallIndex)
	{
	case 0:
		{
			sgFloat rooth_log_decrease_step = houseSizeY/inRoofCount;
			 for (int i=0; i<logsCount+inRoofCount; i++)
			 {
				 if (i==0)
				 {
					 sgC3DObject* log1 = BuildLog(houseSizeY, true, log_size, groove_shift_count* log_size);
					
					 log1->InitTempMatrix()->Rotate(zeroP, xAxe, 90.0*M_PI/180.0);
					 log1->ApplyTempMatrix();
					 log1->DestroyTempMatrix();
					 
					// log1->Triangulate(SG_VERTEX_TRIANGULATION);
					 log1->SetAttribute(SG_OA_COLOR,20);
					 sgGetScene()->AttachObject(log1);

					 m_walls[0].push_back(log1);
				 }
				 else  if (i<logsCount)
				 {
					 sgC3DObject* log_i = (sgC3DObject*)(m_walls[0][0]->Clone());

					 log_i->InitTempMatrix();
					 SG_VECTOR transVect = {0, 0, i*m_logs_shifts};
					 log_i->GetTempMatrix()->Translate(transVect);
					 log_i->ApplyTempMatrix();
					 log_i->DestroyTempMatrix();

					// log_i->Triangulate(SG_VERTEX_TRIANGULATION);
					 log_i->SetAttribute(SG_OA_COLOR,20);
					 sgGetScene()->AttachObject(log_i);

					 m_walls[0].push_back(log_i);
				 }
				 else
				 {
					 sgC3DObject* log1 = BuildLog(houseSizeY-2*(i-logsCount)*rooth_log_decrease_step, false, 0, 0);

					 if (!log1)
						 break;
					 log1->InitTempMatrix()->Rotate(zeroP, xAxe, 90.0*M_PI/180.0);
					 SG_VECTOR transVect = {0, -(i-logsCount)*rooth_log_decrease_step, i*m_logs_shifts};
					 log1->GetTempMatrix()->Translate(transVect);
					 log1->ApplyTempMatrix();
					 log1->DestroyTempMatrix();

					// log1->Triangulate(SG_VERTEX_TRIANGULATION);
					 log1->SetAttribute(SG_OA_COLOR,20);
					 sgGetScene()->AttachObject(log1);

					 m_walls[0].push_back(log1);
				 }
				 
			 }
		}
		break;
	case 1:
		{
			for (int i=0; i<logsCount; i++)
			{
				if (i==0)
				{
					sgC3DObject* log1 = BuildLog(houseSizeX+2*groove_shift_count*log_size, true, log_size, groove_shift_count* log_size);

					log1->InitTempMatrix()->Rotate(zeroP, zAxe, 90.0*M_PI/180.0);
					log1->GetTempMatrix()->Rotate(zeroP, yAxe, 90.0*M_PI/180.0);
					SG_VECTOR transVect = {-groove_shift_count* log_size, -groove_shift_count* log_size, log_size};
					log1->GetTempMatrix()->Translate(transVect);
					log1->ApplyTempMatrix();
					log1->DestroyTempMatrix();

					//log1->Triangulate(SG_VERTEX_TRIANGULATION);
					log1->SetAttribute(SG_OA_COLOR,20);
					sgGetScene()->AttachObject(log1);

					m_walls[1].push_back(log1);
				}
				else
				{
					sgC3DObject* log_i = (sgC3DObject*)(m_walls[1][0]->Clone());

					log_i->InitTempMatrix();
					SG_VECTOR transVect = {0, 0, i*m_logs_shifts};
					log_i->GetTempMatrix()->Translate(transVect);
					log_i->ApplyTempMatrix();
					log_i->DestroyTempMatrix();

					//log_i->Triangulate(SG_VERTEX_TRIANGULATION);
					log_i->SetAttribute(SG_OA_COLOR,20);
					sgGetScene()->AttachObject(log_i);

					m_walls[1].push_back(log_i);
				}

			}
		}
		break;
	case 2:
		{
			for (int i=0;i<m_walls[0].size(); i++)
			{
				sgC3DObject* log_i = (sgC3DObject*)(m_walls[0][i]->Clone());

				log_i->InitTempMatrix();
				SG_VECTOR transVect = {houseSizeX, 0, 0};
				log_i->GetTempMatrix()->Translate(transVect);
				log_i->ApplyTempMatrix();
				log_i->DestroyTempMatrix();

				//log_i->Triangulate(SG_VERTEX_TRIANGULATION);
				log_i->SetAttribute(SG_OA_COLOR,20);
				sgGetScene()->AttachObject(log_i);

				m_walls[2].push_back(log_i);
			}
		}
		break;
	case 3:
		{
			for (int i=0;i<m_walls[1].size(); i++)
			{
				sgC3DObject* log_i = (sgC3DObject*)(m_walls[1][i]->Clone());

				log_i->InitTempMatrix();
				SG_VECTOR transVect = {0, -houseSizeY+2*groove_shift_count*log_size, 0};
				log_i->GetTempMatrix()->Translate(transVect);
				log_i->ApplyTempMatrix();
				log_i->DestroyTempMatrix();

				//log_i->Triangulate(SG_VERTEX_TRIANGULATION);
				log_i->SetAttribute(SG_OA_COLOR,20);
				sgGetScene()->AttachObject(log_i);

				m_walls[3].push_back(log_i);
			}
		}
		break;
	default:
		break;
	}
}

void  HouseModel::BuildWindow(sgFloat houseSizeX,sgFloat houseSizeY)
{
	SG_POINT  arcBeg = {houseSizeX/4, 0, 0};
	SG_POINT  arcEnd = {-houseSizeX/4, 0, 0};
	SG_POINT  arcMid = {0, houseSizeX/6, 0};

	SG_ARC arc1;
	arc1.FromThreePoints(arcBeg, arcEnd, arcMid, false);

	sgCObject*  cont_objcts[4];

	cont_objcts[0] = sgCArc::Create(arc1);
	cont_objcts[1] = sgCreateLine(arcBeg.x, arcBeg.y, arcBeg.z, arcBeg.x, arcBeg.y-houseSizeX/4, arcBeg.z);
	cont_objcts[2] = sgCreateLine(arcBeg.x, arcBeg.y-houseSizeX/4, arcBeg.z, arcEnd.x, arcBeg.y-houseSizeX/4, arcBeg.z);
	cont_objcts[3] = sgCreateLine(arcEnd.x, arcBeg.y-houseSizeX/4, arcBeg.z, arcEnd.x, arcEnd.y, arcEnd.z);

	sgCContour* win_cont = sgCContour::CreateContour(cont_objcts, 4);
	
	SG_VECTOR  extrVect = {0,0, 1000};
	sgC3DObject*  winObj = (sgC3DObject*)sgKinematic::Extrude(*win_cont, NULL, 0, extrVect, true);

	sgDeleteObject(win_cont);

	
	winObj->InitTempMatrix()->Rotate(zeroP, zAxe, 90.0*M_PI/180.0);
	winObj->GetTempMatrix()->Rotate(zeroP, yAxe, 90.0*M_PI/180.0);
	SG_VECTOR transVect = {-houseSizeX/4, -houseSizeY/2, houseSizeX/3};
	winObj->GetTempMatrix()->Translate(transVect);
	winObj->ApplyTempMatrix();
	winObj->DestroyTempMatrix();

	size_t oldCnt = m_walls[0].size();
	for (int i=0;i<oldCnt;i++)
	{
		sgCGroup* boolRes = sgBoolean::Sub(*m_walls[0][i], *winObj);
		if (boolRes)
		{
			int ChCnt = boolRes->GetChildrenList()->GetCount();

			sgCObject**  allChilds = (sgCObject**)malloc(ChCnt*sizeof(sgCObject*));
			boolRes->BreakGroup(allChilds);

			sgCObject::DeleteObject(boolRes);
			
			sgGetScene()->DetachObject(m_walls[0][i]);
			sgDeleteObject(m_walls[0][i]);

			for (int j=0;j<ChCnt;j++)
			{
				//((sgC3DObject*)allChilds[j])->Triangulate(SG_VERTEX_TRIANGULATION);
				sgGetScene()->AttachObject(allChilds[j]);
			}

			m_walls[0][i] = (sgC3DObject*)allChilds[0];

			for (int j=1;j<ChCnt;j++)
				m_walls[0].push_back((sgC3DObject*)allChilds[j]);

			free(allChilds);
		}
		//else
		//	m_walls[0][i]->Triangulate(SG_VERTEX_TRIANGULATION);
	}

	sgDeleteObject(winObj);
}

void  HouseModel::CutForRoof(sgFloat log_size, sgFloat houseSizeX,sgFloat houseSizeY)
{
	sgCObject*  cont_objcts[6];

	cont_objcts[0] = sgCreateLine(0,2*houseSizeY,  0,     0,2*houseSizeY+100,  0);
	cont_objcts[1] = sgCreateLine(0,2*houseSizeY+100,  0,    0,houseSizeY/2,  log_size*(logsCount+inRoofCount)+100);
	cont_objcts[2] = sgCreateLine(0, houseSizeY/2,  log_size*(logsCount+inRoofCount)+100, 0, -houseSizeY-100, 0);
	cont_objcts[3] = sgCreateLine(0,-houseSizeY-100, 0, 0, -houseSizeY, 0);
	cont_objcts[4] = sgCreateLine(0,-houseSizeY,  0,    0, houseSizeY/2, log_size*(logsCount+inRoofCount));
	cont_objcts[5] = sgCreateLine(0, houseSizeY/2, log_size*(logsCount+inRoofCount), 0,2*houseSizeY,  0);

	sgCContour* roof_cont = sgCContour::CreateContour(cont_objcts, 6);

	SG_VECTOR transVect = {-100, -houseSizeY, 30};
	roof_cont->InitTempMatrix()->Translate(transVect);
	roof_cont->ApplyTempMatrix();
	roof_cont->DestroyTempMatrix();


	SG_VECTOR  extrVect = {2*houseSizeX,0, 0};
	sgC3DObject*  roofTmpObj = (sgC3DObject*)sgKinematic::Extrude(*roof_cont, NULL, 0, extrVect, true);

	for (int wallsInd = 0; wallsInd<4; wallsInd++)
	{

			size_t oldCnt = m_walls[wallsInd].size();
			for (int i=0;i<oldCnt;i++)
			{
				sgCGroup* boolRes = sgBoolean::Sub(*m_walls[wallsInd][i], *roofTmpObj);
				if (boolRes)
				{
					int ChCnt = boolRes->GetChildrenList()->GetCount();

					sgCObject**  allChilds = (sgCObject**)malloc(ChCnt*sizeof(sgCObject*));
					boolRes->BreakGroup(allChilds);

					sgCObject::DeleteObject(boolRes);

					sgGetScene()->DetachObject(m_walls[wallsInd][i]);
					sgDeleteObject(m_walls[wallsInd][i]);

					for (int j=0;j<ChCnt;j++)
					{
						((sgC3DObject*)allChilds[j])->Triangulate(SG_VERTEX_TRIANGULATION, SG_SEPARATE_BUFFERS);
						sgGetScene()->AttachObject(allChilds[j]);
					}

					m_walls[wallsInd][i] = (sgC3DObject*)allChilds[0];

					for (int j=1;j<ChCnt;j++)
						m_walls[wallsInd].push_back((sgC3DObject*)allChilds[j]);

					free(allChilds);
				}
				else
					m_walls[wallsInd][i]->Triangulate(SG_VERTEX_TRIANGULATION, SG_SEPARATE_BUFFERS);
			}
	}

	sgDeleteObject(roofTmpObj);
	sgDeleteObject(roof_cont);
}

void  HouseModel::BuildRoof(sgFloat log_size, sgFloat houseSizeX,sgFloat houseSizeY)
{
	sgCObject*  cont_objcts[2];

	cont_objcts[0] = sgCreateLine(0,-100,  m_logs_shifts*(logsCount-4),    0, houseSizeY/2, log_size*(logsCount+inRoofCount));
	cont_objcts[1] = sgCreateLine(0, houseSizeY/2, log_size*(logsCount+inRoofCount), 0,houseSizeY+100, m_logs_shifts*(logsCount-4));
	
	sgCContour* roof_cont = sgCContour::CreateContour(cont_objcts, 2);

	SG_VECTOR transVect = {-100, -houseSizeY, 42};
	roof_cont->InitTempMatrix()->Translate(transVect);
	roof_cont->ApplyTempMatrix();
	roof_cont->DestroyTempMatrix();


	SG_VECTOR  extrVect = {houseSizeX+200,0, 0};
	sgC3DObject*  roofObj = (sgC3DObject*)sgKinematic::Extrude(*roof_cont, NULL, 0, extrVect, true);

    if (roofObj)
    {
        roofObj->Triangulate(SG_VERTEX_TRIANGULATION, SG_SEPARATE_BUFFERS);
        sgGetScene()->AttachObject(roofObj);
    }

	sgDeleteObject(roof_cont);
}

void  HouseModel::Build(sgFloat log_size, sgFloat houseSizeX, sgFloat houseSizeY)
{
	sgC3DObject::AutoTriangulate(false, SG_DELAUNAY_TRIANGULATION, SG_SEPARATE_BUFFERS);
	BuildLogSection(log_size);

	BuildWall(0, log_size, houseSizeX, houseSizeY);
	BuildWall(1, log_size, houseSizeX, houseSizeY);
	BuildWall(2, log_size, houseSizeX, houseSizeY);
	BuildWall(3, log_size, houseSizeX, houseSizeY);

	BuildWindow(houseSizeX, houseSizeY);

	CutForRoof(log_size, houseSizeX,houseSizeY);
	BuildRoof(log_size, houseSizeX,houseSizeY);

	sgDeleteObject(m_log_section);
}